function [P, X] = initialize_P(data, dimP, method, type, tau) 
% This function is used for initialization of the projection P, such that 
% P^T*X'*X is the reduced data.
%
% Input
%       X       --  the training data set stored by cell, in which the c-th
%                   one stands for the data from class c, with size p x Nc;
%       dimP    --  the desired reduced dimension;
%       method  --  the method used for initialization: 
%                       'random'    -- random matrix;
%                       'PCA'       -- PCA on the training data;
%                       'LDA'       -- LDA on the training data;
%
% Output
%       P       -- the initialized projection P;
%       X       -- the data set converted from data.
%
% Author: Shu (Aimery) Kong
% Date:   Jan. 19, 2013

X = cell2mat(data);
C = length(data);
if nargin == 1
    dimP = size(X, 1);
elseif nargin == 2
    method = 'random';
end

if strcmp(method, 'random')
    P = rand(size(X,2), dimP);
    P = P ./ repmat( sqrt(sum(P.^2, 1)), size(P,1), 1);
elseif strcmp(method, 'PCA')
    G = X'*X;
    [P, S, V] = svd(G);
    P = P(:, 1:dimP);
elseif strcmp(method, 'LDA')
    dataNew = data;    
    mc = zeros(size(X,2), C);
    Sw = 0;
    Sb = 0;
    for c = 1:C
        dataNew{c} = kernelFunction(X, data{c}, type, tau);
        mc(:, c) = mean(dataNew{c}, 2);
        tmp = dataNew{c} - repmat( mc(:,c), 1, size(dataNew{c},2) );
        Sw = Sw + tmp*tmp';
    end
    m = mean(cell2mat(dataNew), 2);
    for c = 1:C
        tmp = mc(:,c) - m;
        Sb = Sb + size(dataNew,2)* (tmp*tmp');
    end
    [P, S, V] = svd( pinv(Sw + 0.0001*eye(size(Sw,1))) * Sb);
    P = P(:, 1:dimP);
    
else
    P = rand(size(X,2), dimP);
    P = P ./ repmat( sqrt(sum(P.^2, 1)), size(P,1), 1);
end


